/*
 * Author:
 * Daniel Blake Cogburn scifiboi@yahoo.com
 *
 * Copyright (c) 2013, Daniel Blake Cogburn
 *
 * All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 *		* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *		* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the distribution.
 *		* Neither the name of the [ORGANIZATION] nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *
 *		THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *		"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *		A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *		CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *		EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *		                                     PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *		                                     PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *		LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *		                                                           NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * SIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;
using OpenTK.Audio;
using OpenTK.Audio.OpenAL;
using OpenTK.Input;

namespace BlockEngine
{
	public class Chunk
	{
		public const byte chunkSize = 16;
		private bool Changed = false;
		private byte[,,] chunk;
		private byte CX;
		private byte CY;
		private byte CZ;
		public enum BlockIDs : byte
		{
			Air = 0,
			Stone = 1,
			Dirt_With_Grass = 2,
			Dirt = 3,
			Cobble = 4,
			Log = 5,
			Leaves = 6,
			Water = 7,
			Ice = 8,
			Snow = 9,
		}
		
		public Chunk (long x, long y, long z)
		{
			chunk = new byte[chunkSize, chunkSize, chunkSize];
			Vertices = 0;
			CX = x;
			CY = y;
			CZ = z;
			Changed = true;
			GL.GenBuffers (1, out VBO);
		}
		~Chunk()
		{
			GL.DeleteBuffers (1, ref VBO);
		}
		
		public void Set(int x, int y, int z, byte block)
		{
			chunk[x,y,z] = block;
			Changed = true;
		}
		public byte Get(int x, int y, int z)
		{
			return chunk[x,y,z];
		}
	}
}

