/*
 * Author:
 * Daniel Blake Cogburn scifiboi@yahoo.com
 *
 * Copyright (c) 2013, Daniel Blake Cogburn
 *
 * All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 *		* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *		* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the distribution.
 *		* Neither the name of the [ORGANIZATION] nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *
 *		THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *		"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *		A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *		CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *		EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *		                                     PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *		                                     PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *		LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *		                                                           NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * SIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Drawing;
using System.Windows.Forms;

using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;
using OpenTK.Audio;
using OpenTK.Audio.OpenAL;
using OpenTK.Input;

namespace BlockEngine
{
	public static class Drawing
	{
		public static int chunkSize {get; set;}
		private int vbo;
		public enum BlockIDs
		{
			Air = 0,
			Dirt = 1,
			DirtWithGrass = 2,
			Stone = 3,
			Cobblestone = 4,
			Log = 5,
			Leaves = 6,
			Planks = 7,
			Sand = 8,
			Water = 9,
			Lava = 10
		}
		
		public static void DrawBlockImmediate(int x, int y, int z, float[] color)
		{
			GL.Begin(BeginMode.Quads);
			
			//Render the Z+ face of the cube
			GL.Color3 (1.0f, 0.0f, 0.0f); GL.Vertex3 (x,   y,   z+1);
			GL.Color3 (1.0f, 0.0f, 0.0f); GL.Vertex3 (x+1, y,   z+1);
			GL.Color3 (1.0f, 0.0f, 0.0f); GL.Vertex3 (x+1, y+1, z+1);
			GL.Color3 (1.0f, 0.0f, 0.0f); GL.Vertex3 (x,   y+1, z+1);
			
			//Render the Z- face of the cube
			GL.Color3 (0.0f, 1.0f, 0.0f); GL.Vertex3 (x,   y,   z);
			GL.Color3 (0.0f, 1.0f, 0.0f); GL.Vertex3 (x+1, y,   z);
			GL.Color3 (0.0f, 1.0f, 0.0f); GL.Vertex3 (x+1, y+1, z);
			GL.Color3 (0.0f, 1.0f, 0.0f); GL.Vertex3 (x,   y+1, z);
			
			//Render the X+ face of the cube
			GL.Color3 (0.0f, 0.0f, 1.0f); GL.Vertex3 (x+1, y,   z+1);
			GL.Color3 (0.0f, 0.0f, 1.0f); GL.Vertex3 (x+1, y,   z);
			GL.Color3 (0.0f, 0.0f, 1.0f); GL.Vertex3 (x+1, y+1, z);
			GL.Color3 (0.0f, 0.0f, 1.0f); GL.Vertex3 (x+1, y+1, z+1);
			
			//Render the X- face of the cube
			GL.Color3 (1.0f, 0.0f, 1.0f); GL.Vertex3 (x, y,   z+1);
			GL.Color3 (1.0f, 0.0f, 1.0f); GL.Vertex3 (x, y,   z);
			GL.Color3 (1.0f, 0.0f, 1.0f); GL.Vertex3 (x, y+1, z);
			GL.Color3 (1.0f, 0.0f, 1.0f); GL.Vertex3 (x, y+1, z+1);
			
			//Render the Y+ face of the cube
			GL.Color3 (1.0f, 1.0f, 1.0f); GL.Vertex3 (x+1, y, z+1);
			GL.Color3 (1.0f, 1.0f, 1.0f); GL.Vertex3 (x+1, y, z);
			GL.Color3 (1.0f, 1.0f, 1.0f); GL.Vertex3 (x,   y, z);
			GL.Color3 (1.0f, 1.0f, 1.0f); GL.Vertex3 (x,   y, z+1);
			
			//Render the Y- face of the cube
			GL.Color3 (0.6f, 0.3f, 0.0f); GL.Vertex3 (x+1, y, z+1);
			GL.Color3 (0.6f, 0.3f, 0.0f); GL.Vertex3 (x+1, y, z);
			GL.Color3 (0.6f, 0.3f, 0.0f); GL.Vertex3 (x,   y, z);
			GL.Color3 (0.6f, 0.3f, 0.0f); GL.Vertex3 (x,   y, z+1);
			
			GL.End ();
		}
		public static void DrawChunkImmediate(int x, int y, int z)
		{
			Console.WriteLine ("Chunk Rendering began for chunk {" + x + ", " + y + ", " + z + "}");
			for (int fx = 1; fx != chunkSize+1; fx++)
			{
				for (int fy = 1; fy != chunkSize+1; fy++)
				{
					for (int fz = 1; fz != chunkSize+1; fz++)
					{
						float[] color = {1.0f, 1.0f, 1.0f};
						DrawBlockImmediate (fx + ((x+1)*(chunkSize)), fy + ((y+1)*(chunkSize)), fz + ((z+1)*(chunkSize)), color);
						Console.WriteLine ("Block rendered at position {" + (fx + ((x+1)*(chunkSize))) + ", " + (fy + ((y+1)*(chunkSize))) + ", " + (fz + ((z+1)*(chunkSize))) + "}");
					}
				}
			}
		}
		public static void DrawBlockVBO(int x, int y, int z, byte BlockID)
		{
			Vector3[] Vertices = new Vector3[6*4];
			int[] ColorData = new int[6*4];
			
			//Add the z+ face
			Vertices[0] = new Vector3(x,   y,   z+1);
			Vertices[1] = new Vector3(x+1, y,   z+1);
			Vertices[2] = new Vector3(x,   y+1, z+1);
			Vertices[3] = new Vector3(x+1, y+1, z+1);
			
			//Add the z- face
			Vertices[4] = new Vector3(x,   y,   z);
			Vertices[5] = new Vector3(x+1, y,   z);
			Vertices[6] = new Vector3(x,   y+1, z);
			Vertices[7] = new Vector3(x+1, y+1, z);
			
			//Add the x+ face
			Vertices[8] = new Vector3(x+1,  y,    z);
			Vertices[9] = new Vector3(x+1,  y+1,  z);
			Vertices[10] = new Vector3(x+1, y,    z+1);
			Vertices[11] = new Vector3(x+1, y+1,  z+1);
			
			//Add the x- face
			Vertices[12] = new Vector3(x, y,   z);
			Vertices[13] = new Vector3(x, y+1, z);
			Vertices[14] = new Vector3(x, y,   z+1);
			Vertices[15] = new Vector3(x, y+1, z+1);
			
			//Add the y+ face
			Vertices[16] = new Vector3(x,   y+1,   z);
			Vertices[17] = new Vector3(x+1, y+1,   z);
			Vertices[18] = new Vector3(x+1, y+1,   z+1);
			Vertices[19] = new Vector3(x,   y+1,   z+1);
			
			//Add the y- face
			ertices[20] = new Vector3(x,    y,   z);
			Vertices[21] = new Vector3(x+1, y,   z);
			Vertices[22] = new Vector3(x+1, y,   z+1);
			Vertices[23] = new Vector3(x,   y,   z+1);
		}
		
		
		
	}
}

