/*
 * Author:
 * Daniel Blake Cogburn scifiboi@yahoo.com
 *
 * Copyright (c) 2013, Daniel Blake Cogburn
 *
 * All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 *		* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *		* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the distribution.
 *		* Neither the name of the [ORGANIZATION] nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *
 *		THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *		"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *		A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *		CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *		EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *		                                     PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *		                                     PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *		LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *		                                                           NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * SIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Drawing;
using System.Windows.Forms;

using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;
using OpenTK.Audio;
using OpenTK.Audio.OpenAL;
using OpenTK.Input;

namespace BlockEngine
{
    class Program : GameWindow
    {
		private const float CameraSpeed = 5.0f;
		private const float MoveSpeed = 10.0f;
		private const float HalfPi = (float)Math.PI/2;

		private Matrix4 CameraMatrix = Matrix4.Identity;
		private Vector2 MouseSpeed;
		private Vector2 MouseDelta;
		private Vector2 MousePosition;
		private Vector3 Position = new Vector3(0.0f, 2.0f, 0.0f);
		private float Pitch = 0.0f;
		private float Yaw = 0.0f;
		private Point WindowCenter;
		void OnMouseMove(object sender, MouseMoveEventArgs e)
		{
			MousePosition = new Vector2 (e.X, e.Y);
		}

        /// <summary>Creates a 800x600 window with the specified title.</summary>
		/// <constructor>This line was added just so it folds in my IDE. :3</constructor>
        public Program() : base(800, 600, GraphicsMode.Default, "BlockEngine")
        {
            VSync = VSyncMode.On;
        }

        /// <summary>Load resources here.</summary>
        /// <param name="e">Not used.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            GL.ClearColor(0.2f, 0.3f, 0.6f, 0.0f);
			WindowCenter = new Point (Bounds.Left + Bounds.Width / 2, Bounds.Top + Bounds.Height / 2);
			Cursor.Position = WindowCenter;
			Cursor.Hide ();
			Mouse.Move += new EventHandler<MouseMoveEventArgs> (OnMouseMove);
            GL.Enable(EnableCap.DepthTest);
			Drawing.chunkSize = 8;
        }
        
		/// <summary>
        /// Called when your window is resized. Set your viewport here. It is also
        /// a good place to set up your projection matrix (which probably changes
        /// along when the aspect ratio of your window).
        /// </summary>
        /// <param name="e">Not used.</param>
        protected override void OnResize(EventArgs e)
        {
			base.OnResize(e);

            GL.Viewport(ClientRectangle.X, ClientRectangle.Y, ClientRectangle.Width, ClientRectangle.Height);

            Matrix4 projection = Matrix4.CreatePerspectiveFieldOfView((float)Math.PI / 4, Width / (float)Height, 1.0f, 64.0f);
            GL.MatrixMode(MatrixMode.Projection);
            GL.LoadMatrix(ref projection);
        }

        /// <summary>
        /// Called when it is time to setup the next frame. Add you game logic here.
        /// </summary>
        /// <param name="e">Contains timing information for framerate independent logic.</param>
        protected override void OnUpdateFrame(FrameEventArgs e)
        {
            base.OnUpdateFrame(e);

			if (Keyboard [Key.Escape])
				Environment.Exit (0);
			if (Keyboard[Key.W])
			{
				Position.X += (float)Math.Cos(Yaw) * MoveSpeed * (float)e.Time;
				//Position.Y += (float)Math.Tan(Pitch) * MoveSpeed * (float)e.Time;
				Position.Z += (float)Math.Sin(Yaw) * MoveSpeed * (float)e.Time;
			}
			if (Keyboard[Key.S])
			{
				Position.X -= (float)Math.Cos(Yaw) * MoveSpeed * (float)e.Time;
				//Position.Y -= (float)Math.Tan(Pitch) * MoveSpeed * (float)e.Time;
				Position.Z -= (float)Math.Sin(Yaw) * MoveSpeed * (float)e.Time;
			}
			if (Keyboard[Key.A])
			{
				Position.X -= (float)Math.Cos(Yaw + Math.PI / 2) * MoveSpeed * (float)e.Time;
				Position.Z -= (float)Math.Sin(Yaw + Math.PI / 2) * MoveSpeed * (float)e.Time;
			}
			if (Keyboard[Key.D])
			{
				Position.X += (float)Math.Cos(Yaw + Math.PI / 2) * MoveSpeed * (float)e.Time;
				Position.Z += (float)Math.Sin(Yaw + Math.PI / 2) * MoveSpeed * (float)e.Time;
			}
			if (Keyboard[Key.Space])
			{
				Position.Y += MoveSpeed * (float)e.Time;
			}
			if (Keyboard[Key.LShift] || Keyboard[Key.RShift])
			{
				Position.Y -= MoveSpeed * (float)e.Time;
			}

			MouseDelta = new Vector2(MousePosition.X - PointToClient(WindowCenter).X, MousePosition.Y - PointToClient(WindowCenter).Y);
			Cursor.Position = WindowCenter;

			MouseSpeed.X *= 0.7f;
			MouseSpeed.Y *= 0.7f;
			MouseSpeed.X += MouseDelta.X / 40.0f * (float)e.Time;
			MouseSpeed.Y += MouseDelta.Y / 40.0f * (float)e.Time;

			Yaw += MouseSpeed.X;
			Pitch -= MouseSpeed.Y;

			if (Pitch <= -HalfPi + 0.01f)
				Pitch = -HalfPi + 0.01f;
			if (Pitch >= HalfPi - 0.01f)
				Pitch = HalfPi - 0.01f;
			// Genius

			Vector3 LookAtPoint = new Vector3((float)Math.Cos(Yaw), (float)Math.Tan(Pitch), (float)Math.Sin(Yaw));

			CameraMatrix = Matrix4.LookAt(Position, Position + LookAtPoint, Vector3.UnitY);
        }

        /// <summary>
        /// Called when it is time to render the next frame. Add your rendering code here.
        /// </summary>
        /// <param name="e">Contains timing information.</param>
        protected override void OnRenderFrame(FrameEventArgs e)
        {
            base.OnRenderFrame(e);

			GL.MatrixMode (MatrixMode.Modelview);
			GL.Clear (ClearBufferMask.ColorBufferBit | ClearBufferMask.DepthBufferBit);
			GL.LoadMatrix (ref CameraMatrix);
			GL.PolygonMode (MaterialFace.FrontAndBack, PolygonMode.Line);

			GL.PushMatrix();

			GL.Translate(1.0f * 2.0f, 0.0f, 1.0f * 2.0f);
			GL.Rotate(90, Vector3.UnitX);
			
			Drawing.DrawChunk (0,0,0);
			
			GL.PopMatrix ();
			
            SwapBuffers ();
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // The 'using' idiom guarantees proper resource cleanup.
            // We request 30 UpdateFrame events per second, and unlimited
            // RenderFrame events (as fast as the computer can handle).

            using (Program game = new Program())
            {
                game.Run(30);
            }
        }
    }
}